#ifndef PLTWIN_H
#define PLTWIN_H

#include <qwidget.h>
#include <qprinter.h>

#include "ddplot.h"


class PltWin : public QWidget
{
  Q_OBJECT

public:
  QString FName;
  QColor zColorLayer[MAX_ATOMS][2];  // colors of the atomic layers (0=foreground, 1=background)

  // coordinates of atoms and their relative displacements
  double xyzRel[MAX_ATOMS][3];       // coordinates of atoms in the relaxed configuration
  double xyzInit[MAX_ATOMS][3];      // coordinates of atoms in the initial (unrelaxed) configuration
  double xyzInert[MAX_ATOMS][3];     // coordinates of inert atoms
  double aDisp[MAX_ATOMS][3];        // atomic displacements (real values)
  int atomType[MAX_ATOMS];    // flag recognizing the types of atoms
  int NRel;                // number of atoms in the relaxed block
  int NInit;               // number of atoms in the initial block
  int NInert;              // number of inert atoms (block in the BOP format)

  // parameters of the atomic block
  double csys[3][3];                 // coordinate system of the block
  double blSize[3];        // calculated size of the block in the x,y,z direction
  double xyzMin[3], xyzMax[3];
  double period[3];        // periodicity in the three directions
  char *tail;              // the tail of the .plt file containing the relaxation info (text)
  int TailSize;            // size of the trailing info stored in <tail>

  // Z-layers
  double zCoordLayer[MAX_ATOMS];   // z-coordinates of layers (0 to NumZLayers-1)
  int zLayer[MAX_ATOMS];   // identifiers of the layers
  int NumZLayers;          // number of layers of atoms
  int zLineThickLayer[MAX_ATOMS];  // line thickness of atoms in a specific layer
  int zDiamLayer[MAX_ATOMS];  // diameter of atoms in a specific layer in pixels
  bool zLayerSel[MAX_ATOMS];  // true if a particular Z-layer is selected (considered for plotting arrows)
  bool showZLayers;        // true if the active Z-layers should be visualized as a small inset

  // dislocations
  double xCore,yCore;
  double lenBurgers;       // length of the Burgers vector

  // grain boundaries
  double gbYcoor;          // y-position of the grain boundary
  int gbLayer[MAX_ATOMS];  // grain boundary layer in which the atom lies(upper grain>0, lower grain<0)
  bool isGBfile;           // true if the file contains a grain boundary data
  bool showGB;             // true if the grain boundary is being plotted 

  // appearance of the plot
  double ProjVector[3];    // vector into whose direction the displacements are to be projected
                           // in the system [1-21], [-101], [111]
  double arrAngle;         // angle of arrows
  double ZFact;            // zoom factor
  double xPan,yPan;        // panning
  double DScaleFact;       // scaling factor for displacements (arrow lengths)
  double AtomDispScale;    // scaling of atomic displacements in the relaxed configuration
  double factor;
  int atomScreen[MAX_ATOMS][2];    // coordinates of atoms on the screen [pixels]
  int xBorder,yBorder,xOffset,yOffset;
  int xyzCen[3];
  int shortestArrow;    // shortest arrow to be plotted (in pixels)
  int thicknessArrow;   // thickness of an arrow (in pixels)
  int plotType;               // type of plot
  int DispComponent;          // component of the displacements plotted
  int AtomPos;             // plotting of atomic positions (RELAXED, UNRELAXED)  
  int PTAngle;             // angle between two neighboring traces of planes plotted
  bool PrintMagDisp;       // print magnitudes of displacements (value/a) ?
  bool AtomNumbers;        // print the numbers of atoms
  bool isSelected;         // true if this plot is selected
  bool DisloCenter;        // show the dislocation center ?
  bool PlaneTraces;        // show the traces of the planes ?
  bool InertAtoms;         // plot inert atoms ?

  // radial distribution function
  double RdfDist[MAX_NEIGHBORS];  // distances of the first, second, etc., nearest neighbors
  int arrNeighNum;         // number of different neighbor distances to be considered
  int arrNeighbors[MAX_NEIGHBORS];  // which neighbor distances (1=first, 2=second, etc.)      
  int RdfNum[MAX_NEIGHBORS];        // numbers of 1st, 2nd, 3rd, ..., nearest neighbors
  int RdfNNeigh;           // how many nearest neighbors ? (3=first, second and third)

  // linked neighbor list
  double rcut;             // cut-off radius (minimum size of an edge in either direction)
  double cellsize[3];      // size of a cell's edge in x, y and z direction
  int ncell[3];            // number of cells in x, y and z direction
  int list[MAX_ATOMS];     // linked neighbor list
  int head[MAX_CELL][MAX_CELL][MAX_CELL];  // index of a head atom in each cell
  int NeighListUnrel[MAX_ATOMS][MAX_NEIGHBORS];


  explicit PltWin( QWidget *parent=0, const char *name=0, WFlags f=0 );
  ~PltWin();
  void paintEvent( QPaintEvent * );
  void plotEdgeComponent(QPainter *p);
  void plotScrewComponent(QPainter *p);

protected:
  virtual void resizeEvent( QResizeEvent * );

public:
  void drawIt(QPainter *p);
  void printIt(QPrinter *printer);

  bool DrawArrow( QPainter *p, double x0, double y0, double x1, double y1 );
  void DrawCircle( QPainter *p, int atom, double x, double y, int dia);
  void DrawLine( QPainter *p, double x0, double y0, double x1, double y1, int addx, int addy);
  void DrawPlaneTraces( QPainter *p, double x0, double y0, double x1, double y1 );
  void DrawText( QPainter *p, double x, double y, QString txt );
  void Pan( double xsteps, double ysteps );
  void SetGeometry();
  void ShowActiveZLayers( QPainter *p );
  void Zoom( double zfact );

public:
  int FindNearestAtom( int x, int y );
  bool LoadBOP( QString fname );
  bool LoadDD( QString fname );
  bool LoadGB( QString fname, long int *blstart, PltWin *initConf );
  bool LoadPLT( QString fname );
  void PrepareBlock();
};

#endif
